# An example MIPSbot program that shows controlling MIPSbot's velocity
# and orientation.
# Ken Vollmar Aug. 13, 2005
# Adapted from "SPIMbot"
# http://www-courses.cs.uiuc.edu/~cs232/assignments/spimbot_ex.s
# by Craig Zilles, http://www-faculty.cs.uiuc.edu/~zilles/

.data
datapts: .word
    # Start Texas datapoints
    0, 107, 111,   # Don't leave a track while headed to this point
    1, 157, 110,   # Do leave ...
    1, 157, 146,
    1, 204, 161,
    1, 257, 163,
    1, 266, 235,
    1, 208, 277,
    1, 210, 317,
    1, 174, 302,
    1, 123, 238,
    1, 101, 253,
    1, 41, 192,
    1, 103, 194,
    1, 107, 111,  # End Texas datapoints,

    0, 95, 40,  # Start "SIGCSE" datapoints. Start S
    1, 50, 40,
    1, 50, 55,
    1, 95, 75, # slant of S
    1, 95, 90,
    1, 50, 90,  # end S
    0, 110, 90, # I
    1, 110, 40,
    0, 165, 40, # start G
    1, 120, 40,
    1, 120, 90,
    1, 165, 90,
    1, 165, 70,
    1, 150, 70, # end G
    0, 220, 40, # start C
    1, 175, 40,
    1, 175, 90,
    1, 220, 90, # end C
    0, 230, 90,
    1, 275, 90, # start S
    1, 275, 75,
    1, 230, 55,
    1, 230, 40,
    1, 275, 40, # end S
    0, 330, 40,
    1, 285, 40, # start E
    1, 285, 90,
    1, 330, 90,
    0, 320, 65,
    1, 285, 65, # end E

    0, 275, 270,  # Start "2006" datapoints, the 2 of 2006
    1, 295, 270,  # top of the 2
    0, 300, 270,  # first 0
    1, 320, 270,  # top of the first 0
    0, 325, 270,  # second 0
    1, 345, 270,  # top of 2nd 0
    0, 350, 270,  # 6
    1, 370, 270,  # top of 6
    0, 370, 285,  #
    1, 370, 300,  # right side of 6
    1, 350, 300,  # bottomm of 6
    0, 345, 300,
    1, 325, 300,  # bottom of 2nd 0
    0, 320, 300,
    1, 300, 300,  # bottom of 1st 0
    0, 295, 300,
    1, 275, 300,  # bottom of 2
    1, 275, 295,
    1, 295, 275,
    1, 295, 270,  # 2 is now finished
    0, 300, 270,
    1, 300, 300, # left side of 1st 0
    0, 320, 300,
    1, 320, 270, # right side of 1st 0
    0, 325, 270,
    1, 325, 300, # left side of 2nd 0
    0, 345, 300,
    1, 345, 270, # right side of 2nd 0
    0, 350, 270,
    1, 350, 300, # left side of 6
    1, 350, 285,
    1, 370, 285, # finished 6
    
    0, 400, 150,  # move cursor off the logo

    0, -1, -1   # Flag for finished with data points
.text

  # Register usage:
  # $s0 Xs: X start value for current segment
  # $s1 Xc: X current value for current segment
  # $s2 Xg: X goal value for current segment (end point)
  # $s3 Ys: Y start value for current segment
  # $s4 Yc: Y current value for current segment
  # $s5 Yg: Y goal value for current segment (end point)
  # $s6 Boolean: leave a track from (Xs,Ys) to (Xg,Yg) 1:true, 0:false
  # $s7 Address of 3-value point set from memory

  # $a0 -- address for memory-mapped IO heading value  0xffff8010
  # $a1 -- address for memory-mapped IO "leave a track" value  0xffff8020;
  # $a2 -- address for memory-mapped IO "Where are we X" value   0xffff8030;
  # $a3 -- address for memory-mapped IO "Where are we Y" value    0xffff8040;
  # $v0 -- address for memory-mapped IO "Move" value    0xffff8050;
  lui $a0, 0xffff
  ori $a0, $a0, 0x8010
  addi $a1, $a0, 0x10
  addi $a2, $a1, 0x10
  addi $a3, $a2, 0x10
  addi $v0, $a3, 0x10
  addi $t1, $zero, 1  # Known value for "true" for boolean values

  lw $s0, 0($a2)  # Find initial bot (X,Y) location
  lw $s3, 0($a3)
  
  la $s7, datapts
  
logotop:

  lw $s6, 0($s7) #  \
  lw $s2, 4($s7) #   > Three values for this line segment
  lw $s5, 8($s7) #  /
  
  ble $s2, $zero, logoend  # Flag for end of data is a negative X datapoint
  
  addi $s7, $s7, 12  # increment address by three words for next set of data points
  
  sw $s6, 0($a1) # Set leave-track MMIO value
moveX:
  beq $s0, $s2, moveY # if (Xs == Xg) No X movement
  blt $s0, $s2, elseX # if (Xs < Xg) goto else part
    # Here we know that (Xs > Xg), so X movement will be 270 degrees.
    li	$t0, 270   # angle of movement
    sw $t0, 0($a0) # Set angle-movement MMIO value
    sw $t1, 0($v0) # Set "move/don't move" MMIO value to "move"
    busyX1:
      lw $s1, 0($a2)  # Read X current value
      ble $s1, $s2, moveY # if (Xc <= Xg) X movement is finished
      j busyX1
  elseX:
    # Here we know that (Xs <= Xg), so X movement will be 90 degrees.
    li	$t0, 90   # angle of movement
    sw $t0, 0($a0) # Set angle-movement MMIO value
    sw $t1, 0($v0) # Set "move/don't move" MMIO value to "move"
    busyX2:
      lw $s1, 0($a2)  # Read X current value
      bge $s1, $s2, moveY # if (Xc >= Xg) X movement is finished
      j busyX2

moveY:
  beq $s3, $s5, loopend # if (Ys == Yg) No Y movement
  blt $s3, $s5, elseY # if (Ys < Yg) goto else part
    # Here we know that (Ys > Yg), so Y movement will be 0 degrees.
    li	$t0, 0   # angle of movement
    sw $t0, 0($a0) # Set angle-movement MMIO value
    sw $t1, 0($v0) # Set "move/don't move" MMIO value to "move"
    busyY1:
      lw $s4, 0($a3)  # Read Y current value
      ble $s4, $s5, loopend # if (Yc <= Yg) Y movement is finished
      j busyY1
  elseY:
    # Here we know that (Ys <= Yg), so Y movement will be 180 degrees.
    li	$t0, 180   # angle of movement
    sw $t0, 0($a0) # Set angle-movement MMIO value
    sw $t1, 0($v0) # Set "move/don't move" MMIO value to "move"
    busyY2:
      lw $s4, 0($a3)  # Read Y current value
      bge $s4, $s5, loopend # if (Yc >= Yg) Y movement is finished
      j busyY2

loopend:
  sw $zero, 0($a1) # clear leave-track MMIO value
  add $s0, $zero, $s2  # Update for next loop: prev. X goal becomes next X start
  add $s3, $zero, $s5  # Update for next loop: prev. Y goal becomes next Y start
  j logotop

logoend:
  sw $zero, 0($v0) # Set "move/don't move" MMIO value to "don't move"



